<?php
/**
 * 2020 GARWEN
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Garwen to newer
 * versions in the future. If you wish to customize Garwen for your
 * needs please refer to http://www.garwen.fr for more information.
 *
 * @author    Garwen <contact@garwen.fr>
 * @copyright 2020 Garwen
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

class GarwenActionOrderStatusPostUpdateController
{
    public function __construct($module, $file, $path)
    {
        $this->file    = $file;
        $this->module  = $module;
        $this->context = Context::getContext();
        $this->_path   = $path;
    }

    public function cleanProductDetail($products)
    {
        $clean_products = [];
        foreach ($products as $product_order) {
            $image    = Image::getCover($product_order['product_id']);
            $image_id = $image['id_image'];

            $lang_id           = (int)Configuration::get('PS_LANG_DEFAULT');
            $description       = '';
            $description_short = '';
            $product_url       = '';
            try {
                $product           = new Product($product_order['product_id'], false, $lang_id);
                $description       = $product->description;
                $description_short = $product->description_short;
                $product_url       = $this->context->link->getProductLink($product);
            } catch (PrestaShopDatabaseException $e) {
            } catch (PrestaShopException $e) {
                echo $e->getMessage();
            }

            $clean_products[] = [
                'name'              => $product_order['product_name'],
                'description'       => $description,
                'description_short' => $description_short,
                'quantity'          => $product_order['product_quantity'],
                'reference'         => $product_order['product_reference'],
                'unit_price'        => $product_order['unit_price_tax_incl'],
                'image_link'        => $this->context->link->getImageLink(
                    $product_order['product_name'],
                    $image_id,
                    ImageType::getFormattedName('home')
                ),
                'url'               => $product_url,
            ];
        }

        return $clean_products;
    }

    public function run($params)
    {
        //Check if it's the right moment to send data to Garwen
        $order_status = $params['newOrderStatus'];
        if (1 == $order_status->paid && 1 == $order_status->invoice) {
            $customer = new Customer((int)$params['cart']->id_customer);

            //Get Invoice for Garwen X
            $order = new Order((int)$params['id_order']);

            if (Validate::isLoadedObject($order)) {
                $context = Context::getContext();
                $invoice = $order->getInvoicesCollection();
                Hook::exec('actionPDFInvoiceRender', [ 'order_invoice_list' => $invoice ]);
                $pdf        = new PDF($invoice, PDF::TEMPLATE_INVOICE, $context->smarty);
                $pdfContent = $pdf->render(false);

                $options                     = garwen::getOptionAPI();
                $multipart                   = '--------------------------' . microtime(true);
                $options['http']['header'][] = 'Content-Type: multipart/form-data; boundary=' . $multipart;

                $content = "--" . $multipart . "\r\n" .
                    "Content-Disposition: form-data; name=\"invoice\"; filename=\"" .
                    basename($pdf->filename) .
                    "\"\r\n" .
                    "Content-Type: application/pdf\r\n\r\n" .
                    $pdfContent . "\r\n";

                $content .= "--" . $multipart . "\r\n" .
                    "Content-Disposition: form-data; name=\"customer_email\"\r\n\r\n" .
                    $customer->email . "\r\n";

                $currency           = Configuration::get('PS_CURRENCY_DEFAULT');

                $content .= "--" . $multipart . "\r\n" .
                    "Content-Disposition: form-data; name=\"total_paid\"\r\n\r\n" .
                    $order->getTotalPaid($currency) . "\r\n";

                $content .= "--" . $multipart . "\r\n" .
                    "Content-Disposition: form-data; name=\"customer_products\"\r\n\r\n" .
                    json_encode($this->cleanProductDetail($order->getProductsDetail())) . "\r\n";

                $content .= "--" . $multipart . "--\r\n";

                $options['http']['content'] = $content;

                $context = stream_context_create($options);

                Tools::file_get_contents(garwen::GARWEN_API_URL . 'new/sale', false, $context);
            }
        }
    }
}
