<?php
/**
 * 2020 GARWEN
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Garwen to newer
 * versions in the future. If you wish to customize Garwen for your
 * needs please refer to http://www.garwen.fr for more information.
 *
 * @author    Garwen <contact@garwen.fr>
 * @copyright 2020 Garwen
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class Garwen extends Module
{
    protected $config_form = false;
    const GARWEN_API_URL = 'https://app.garwen.fr/api/store/';

//        const GARWEN_API_URL = 'http://127.0.0.1/garwen-core/public/api/store/';

    public function __construct()
    {
        $this->name          = 'garwen';
        $this->tab           = 'billing_invoicing';
        $this->version       = '0.0.1';
        $this->author        = 'Jérémy PASTOURET';
        $this->need_instance = 1;
        $this->module_key    = 'de0bbc03f40886b994d848116a0b0bbd';

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName =
            $this->l('Interconnexion avec Garwen - donnez plus de visibilité et de valeur à vos produits');

        $this->description = $this->l(
            'Ce module envoi automatiquement les factures/produits achetés sur l\'espace Garwen personnel du client'
        );

        $this->confirmUninstall = $this->l('Etes-vous sur de débrancher la connexion entre votre boutique et Garwen ?');
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        Configuration::updateValue('GARWEN_SEND_INVOICE_AUTO', false);
        Configuration::updateValue('GARWEN_TOKEN_API', '');

        include dirname(__FILE__) . '/sql/install.php';

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('backOfficeHeader') &&
            //My Add
            $this->registerHook('actionOrderStatusPostUpdate');
    }

    public function uninstall()
    {
        Configuration::deleteByName('GARWEN_TOKEN_API');
        Configuration::deleteByName('GARWEN_SEND_INVOICE_AUTO');

        include dirname(__FILE__) . '/sql/uninstall.php';

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        $submit = (bool)Tools::isSubmit('submit_garwen_api_form');
        if (true == $submit) {
            //If Garwen is activated but token is null
            if (true === empty(Tools::getValue('GARWEN_TOKEN_API'))
                && true == Tools::getValue('GARWEN_SEND_INVOICE_AUTO')
            ) {
                $this->context->smarty->assign('confirmation', 'token_missing');
                $this->getSendValues();
            } else {
                //if Garwen is desactived
                if (true === empty(Tools::getValue('GARWEN_TOKEN_API'))
                    && false == Tools::getValue('GARWEN_SEND_INVOICE_AUTO')
                ) {
                    $this->postProcess();
                    $this->getConfigValues();
                } else {
                    //Check Token
                    $options = self::getOptionAPI(Tools::getValue('GARWEN_TOKEN_API'));
                    $context = stream_context_create($options);
                    $result  = Tools::file_get_contents(self::GARWEN_API_URL . 'check_token', false, $context);
                    $result  = json_decode($result);
                    if (false === isset($result->message) && '' !== $result->message) {
                        //Save if token is ok
                        $this->postProcess();
                        $this->context->smarty->assign('name', $result->store_name);
                        $this->context->smarty->assign('confirmation', 'ok');
                        $this->getConfigValues();
                    } else {
                        if ('' === $result->message) {
                            $result->message = '404';
                        }
                        $this->context->smarty->assign('confirmation', $result->message);
                        $this->getSendValues();
                    }
                }
            }
        } else {
            $this->getConfigValues();
        }

        $this->context->smarty->assign('module_dir', $this->_path);
        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');

        return $output;
    }

    public static function getOptionAPI($sToken = null)
    {
        if (true === is_null($sToken)) {
            $sToken = Configuration::get('GARWEN_TOKEN_API');
        }

        return [
            'http' => [
                'method'        => 'POST',
                'header'        => [
                    'Accept: application/json',
                    'Authorization: Bearer ' . $sToken,
                ],
                'ignore_errors' => true
            ]
        ];
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return [
            'GARWEN_TOKEN_API'         => Configuration::get('GARWEN_TOKEN_API'),
            'GARWEN_SEND_INVOICE_AUTO' => Configuration::get('GARWEN_SEND_INVOICE_AUTO')
        ];
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    /**
     * Get form data
     */
    protected function getConfigValues()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            $this->context->smarty->assign($key, Configuration::get($key));
        }
    }

    /**
     * Get Send data to view
     */
    protected function getSendValues()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            $this->context->smarty->assign($key, Tools::getValue($key));
        }
    }

    public function getHookController($hook_name)
    {
        // Include the controller file
        include_once dirname(__FILE__) . '/controllers/hook/' . $hook_name . '.php';

        // Build the controller name dynamically
        $controller_name = $this->name . $hook_name . 'Controller';

        // Instantiate controller
        $controller = new $controller_name($this, __FILE__, $this->_path);

        // Return the controller
        return $controller;
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('module_name') == $this->name) {
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path . '/views/js/front.js');
        $this->context->controller->addCSS($this->_path . '/views/css/front.css');
    }

    public function hookActionOrderStatusPostUpdate($params)
    {
        $controller = $this->getHookController('actionOrderStatusPostUpdate');
        return $controller->run($params);
    }
}
